pub enum DrawCommand {
Clear {
color: Color,
},
DrawCircle {
x: f32,
y: f32,
radius: f32,
color: Color,
},
DrawRect {
x: f32,
y: f32,
w: f32,
h: f32,
color: Color,
},
DrawLine {
x1: f32,
y1: f32,
x2: f32,
y2: f32,
thickness: f32,
color: Color,
},
DrawText {
text: String,
x: f32,
y: f32,
size: f32,
color: Color,
},
SetCamera {
camera: Camera,
},
DrawTriangle {
x1: f32,
y1: f32,
x2: f32,
y2: f32,
x3: f32,
y3: f32,
color: Color,
},
DrawPoly {
points: Vec<Vec2>,
color: Color,
},
DrawEllipse {
x: f32,
y: f32,
rx: f32,
ry: f32,
color: Color,
},
DrawRing {
x: f32,
y: f32,
inner_radius: f32,
outer_radius: f32,
color: Color,
},
DrawArc {
x: f32,
y: f32,
radius: f32,
start_angle: f32,
end_angle: f32,
color: Color,
},
}Expand description
Drawing operations that the game can issue each frame.
In the real rendering backend, these would be batched and sent to the GPU. For the API definition, we store the draw commands for deferred rendering.
Variants
Clear
DrawCircle
DrawRect
DrawLine
DrawText
SetCamera
DrawTriangle
DrawPoly
DrawEllipse
DrawRing
DrawArc